<?php
    /*
     * Plugin Name: In8Sync Sales Orders
     * Description: This plugin manages order syncing with NetSuite
     * Version: 3.3.0
     * Author: In8Sync
     * Author URI: http://www.in8sync.com
     */

    /**
     * This is main plugin file which handles plugin installation and removal.
     * It includes required plugin files.
     */
    if (!defined('ABSPATH')) {
        exit; // Exit if accessed directly
    }

//debugging function
    if (!function_exists('in8Pr')) {

        function in8Pr($data) {
            echo "<pre>";
            print_r($data);
            echo "</pre>";
        }

    }

// run the install scripts upon plugin activation
    register_activation_hook(__FILE__, 'installIn8SalesOrderPlugin');

// function for creating log table "in8sync_log"
    function installIn8SalesOrderPlugin() {
        //check if in8sync plugin is installed and activated
        if (!in_array('in8sync/in8sync.php', apply_filters('active_plugins', get_option('active_plugins')))) {
            //deactivate plugin if parent in8sync is not activated
            deactivate_plugins(__FILE__);
            echo '<div id="error" class="error notice is-dismissible"><p>Plugin can\'t be activated.Please install main in8 plugin before activating.</div>';
            die;
        }

        //check if in8sync plugin is installed and activated
        if (!in_array('in8sync-customers/in8sync-customers.php', apply_filters('active_plugins', get_option('active_plugins')))) {
            //deactivate plugin if parent in8sync is not activated
            deactivate_plugins(__FILE__);
            echo '<div id="error" class="error notice is-dismissible"><p>Plugin can\'t be activated.Please install main in8 customer before activating.</div>';
            die;
        }
    }

//plugins load hook
    add_action('plugins_loaded', 'init_in8_sales_sync_plugin', 20);

//intialization function 
    function init_in8_sales_sync_plugin() {
        //check if in8sync and in8sync customers plugin is installed and activated
        $active_plugins = apply_filters('active_plugins', get_option('active_plugins'));

        //check if in8sync and in8sync customers plugin is installed and activated
        if (in_array('in8sync/in8sync.php', $active_plugins) && in_array('in8sync-customers/in8sync-customers.php', $active_plugins)) {

            define('In8_SalesOrder_URL', plugin_dir_url(__FILE__));
            define('In8_SalesOrder_PATH', plugin_dir_path(__FILE__));
            //admin setting tabs for customers and sale orders
            require_once('inc/salesorder.php'); //salesorder settings option tab


            if (in_array('in8sync-purchaseorder/in8sync-purchaseorder.php', $active_plugins)) {
                $purchase_order_conf = get_option('In8_PurchaseOrder_CO', []);
                if (!empty($purchase_order_conf) && isset($purchase_order_conf['all_purchase_orders'])) {
                    return;
                }
            }
	
	        require_once('inc/endpoints.php'); //load order endpoints
			
	        //check if sync enabled from admin conf option
            $co_options = In8_SalesOrder::getOptions(2);
	        $disableSync = IN8_Settings::isSyncDisabled();
	        if (isset($co_options['enable_so_sync']) && $co_options['enable_so_sync'] == 'on' && !$disableSync) {
                require_once('inc/sync.php'); //file for handling customer and order sync
                require_once('inc/subscription-queue.php'); //file for handling subscription queue
                require_once('inc/options.php'); //file for handling customer and order sync
                require_once('inc/functions.php'); //file for handling customer and order sync
                require_once('inc/admin-manual-sync.php'); //admin button for sync
            }
        } else {
            include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
            //deactivate plugin if parent in8sync is not activated
            deactivate_plugins(__FILE__);
        }
    }

//    add_action('admin_init', 'in8_activate_updates_salesorder');

    function in8_activate_updates_salesorder() {
        require_once (IN8_MAIN_DIR . '/code/in8-update.php');
        $plugin_data = get_plugin_data(__FILE__);
        if (isset($plugin_data['Version'])) {
            $plugin_current_version = $plugin_data['Version'];
        } else {
            $plugin_current_version = '2.0.1';
        }
        $plugin_remote_path = 'https://secure.in8sync.com/update/currentversion.php';
        $plugin_slug = plugin_basename(__FILE__);
        new in8_wp_auto_update($plugin_current_version, $plugin_remote_path, $plugin_slug);
    }

    // Add Variation Settings
    add_action('woocommerce_product_after_variable_attributes', 'in8_variation_settings_fields', 10, 3);
    // Save Variation Settings
    add_action('woocommerce_save_product_variation', 'in8_save_variation_settings_fields', 10, 2);

    /**
     * Create new fields for variations
     *
     */
    function in8_variation_settings_fields($loop, $variation_data, $variation) { // Text Field
        woocommerce_wp_text_input(array('id' => 'internalid[' . $variation->ID . ']', 'label' => __('Internal Id', 'woocommerce'), 'description' => __('Internal Id Required', 'woocommerce'), 'value' => get_post_meta($variation->ID, 'internalid', true)));
    }

    /**
     * Save new fields for variations
     *
     */
    function in8_save_variation_settings_fields($post_id) {
        // Text Field
        $text_field = $_POST['internalid'][$post_id];
        if (!empty($text_field)) {
            update_post_meta($post_id, 'internalid', esc_attr($text_field));
        }
    }

    // Display Internal Id field on product general tab
    add_action('woocommerce_product_options_general_product_data', 'in8_add_internalid_meta_to_product_general_tab');

    function in8_add_internalid_meta_to_product_general_tab() {
        global $woocommerce, $post;
        echo '<div class="options_group">';
        ?>
        <p class="form-field internalid[<?php echo $post->ID; ?>]_field ">
            <label for="internalid[<?php echo $post->ID; ?>]">NetSuite Internal ID</label>
            <input type="text" class="short" style="" name="internalid[<?php echo $post->ID; ?>]" id="internalid[<?php echo $post->ID; ?>]" value="<?php echo get_post_meta($post->ID, 'internalid', true); ?>" placeholder="">
        </p>
        <?php
        echo '</div>';
    }

    // Save internal id
    add_action('woocommerce_process_product_meta', 'in8_add_internalid_meta_to_product_general_tab_save');

    function in8_add_internalid_meta_to_product_general_tab_save($post_id) {
        $woocommerce_text_field = $_POST['internalid'][$post_id];
	    if (isset($_POST['internalid'])) {
            update_post_meta($post_id, 'internalid', esc_attr($woocommerce_text_field));
        }
    }
	
	// Display Internal Id field on coupon general tab
	add_action('woocommerce_coupon_options', function() {
		global $woocommerce, $post;
		echo '<div class="options_group">';
		?>
		<p class="form-field internalId[<?php echo $post->ID; ?>]_field ">
			<label for="internalId[<?php echo $post->ID; ?>]">NetSuite Internal ID</label>
			<input type="text" class="short" style="" name="internalId[<?php echo $post->ID; ?>]" id="internalId[<?php echo $post->ID; ?>]" value="<?php echo get_post_meta($post->ID, 'internalId', true); ?>" placeholder="">
		</p>
		<?php
		echo '</div>';
	} );
	
	
	// Save internal id
	add_action('woocommerce_process_shop_coupon_meta', 	function($post_id) {
		$woocommerce_text_field = $_POST['internalId'][$post_id];
		if (!empty($woocommerce_text_field)) {
			update_post_meta($post_id, 'internalId', esc_attr($woocommerce_text_field));
		}
	});
	