<?php
	
	class in8_order_items extends WC_API_Resource {
		
		
		protected $base = '/order_items';
		public function register_routes( $sroutes ) {
			
			# GET /Update Order Item details and recalculate.
			
			$sroutes[ $this->base ] = [
				[ [ $this, 'order_items_update' ], WC_API_Server::READABLE | WC_API_Server::EDITABLE | WC_API_Server::ACCEPT_DATA ],
			];
			
			return $sroutes;
		}
		
		public function order_items_update() {
			
			error_reporting(-1);
			// They are looking up data
			$ordId        = $_GET["id"];
			
			$data = $_REQUEST['data'];
			$data = json_decode( base64_decode( $data ), TRUE );
			$lines = $data["lines"];
			
			
			if ( empty($ordId) ) {
				echo json_encode( [ "errors" => "Missing id argument. POST" ] );
				exit;
			}
			
			# check if the lines are set
			if( empty($lines) ){
				echo json_encode( [ "errors" => "You're not sending lines to save. POST" ] );
				exit;
			}
			
			# get order
			$order = new WC_Order($ordId);
			/* @var WC_Order $order */
			if( empty($order->post) || $order->post->post_type != 'shop_order' ){
				echo json_encode( [ "errors" => "Order doesn't exist. POST" ] );
				exit;
			}
			
			$acceptedLineTypes = [
				'line_item',
//				'shipping',
				'fee',
				'coupon',
			];
			foreach($acceptedLineTypes as $type){
				foreach($order->get_items($type) as $id => $item){
					wc_delete_order_item( $id );
				}
			}
			
			foreach($lines as $line) {
				if( !isset($line['line_type']) ){
					$type = 'line_item';
				}else{
					if( !in_array($line['line_type'], $acceptedLineTypes) ){
						# we don't accept this line type (should we kill the process?)
						continue;
					}
					$type = $line['line_type'];
				}
				
				$helper = new WC_API_Orders_Helper();
				$helper->set_item($order, $type, $line, 'create');
				
			}
			
			// calculate totals and set them
			$order->calculate_totals();
			
			echo json_encode( [ "success" => "Items saved" ] );
			exit;
		}
		
	}
	
	class WC_API_Orders_Helper extends WC_API_Orders{
		public function __construct(){
		
		}
		public function set_item( $order, $item_type, $itemData, $action ) {
			$genericItemId = 0;
			if( class_exists('In8_Customers_Sales_Options') ){
				$genericItemId = In8_Customers_Sales_Options::getGenericItem();
			}
			
			if( $item_type == 'line_item' && ((!empty($itemData['product_id']) && $genericItemId == $itemData['product_id']) || apply_filters('in8_order_save_generic_item_check', false, $itemData, $order)) ){
				$item = new WC_Order_Item_Product();
				if( isset($itemData['id']) ){
					$item = $order->get_item($itemData['id']);
				}
				$item->set_props( $itemData );
				
				$product = wc_get_product($itemData['product_id']);
				$item->set_product($product);
				$item->set_order_id( $order->get_id() );
				$item->set_props( $itemData );
				
				// save product info on the item when it is a generic item
				$genericItemTitle = !empty($itemData['description']) ? $itemData['description'] : $itemData['displayname'];
				if( !empty($genericItemTitle) ) {
					$item->set_name($genericItemTitle);
				}
				$item->add_meta_data('internalid', $itemData['internalid']);
				$item->add_meta_data('sku', $itemData['_sku']);
				
				$item = apply_filters('in8_order_save_generic_item', $item, $itemData, $order->get_id());
				$item->save();
				
				if( !isset($itemData['id']) ){
					$order->add_item($item);
				}
			}else {
				parent::set_item( $order, $item_type, $itemData, $action );
			}
		}
		
		function set_line_item( $order, $item, $action ) {
			$creating  = ( 'create' === $action );
			
			// product is always required
			if ( ! isset( $item['product_id'] ) && ! isset( $item['sku'] ) ) {
				throw new WC_API_Exception( 'woocommerce_api_invalid_product_id', __( 'Product ID or SKU is required', 'woocommerce' ), 400 );
			}
			
			// when updating, ensure product ID provided matches
			if ( 'update' === $action ) {
				
				$item_product_id   = wc_get_order_item_meta( $item['id'], '_product_id' );
				$item_variation_id = wc_get_order_item_meta( $item['id'], '_variation_id' );
				
				if ( $item['product_id'] != $item_product_id && $item['product_id'] != $item_variation_id ) {
					throw new WC_API_Exception( 'woocommerce_api_invalid_product_id', __( 'Product ID provided does not match this line item', 'woocommerce' ), 400 );
				}
			}
			
			if ( isset( $item['product_id'] ) ) {
				$product_id = $item['product_id'];
			} elseif ( isset( $item['sku'] ) ) {
				$product_id = wc_get_product_id_by_sku( $item['sku'] );
			}
			
			// variations must each have a key & value
			$variation_id = 0;
			if ( isset( $item['variations'] ) && is_array( $item['variations'] ) ) {
				foreach ( $item['variations'] as $key => $value ) {
					if ( ! $key || ! $value ) {
						throw new WC_API_Exception( 'woocommerce_api_invalid_product_variation', __( 'The product variation is invalid', 'woocommerce' ), 400 );
					}
				}
				$variation_id = $this->get_variation_id( wc_get_product( $product_id ), $item['variations'] );
			}
			
			$product = wc_get_product( $variation_id ? $variation_id : $product_id );
			
			// must be a valid WC_Product
			if ( ! is_object( $product ) ) {
				throw new WC_API_Exception( 'woocommerce_api_invalid_product', __( 'Product is invalid.', 'woocommerce' ), 400 );
			}
			
			// quantity must be positive float
			if ( isset( $item['quantity'] ) && floatval( $item['quantity'] ) <= 0 ) {
				throw new WC_API_Exception( 'woocommerce_api_invalid_product_quantity', __( 'Product quantity must be a positive float.', 'woocommerce' ), 400 );
			}
			
			// quantity is required when creating
			if ( $creating && ! isset( $item['quantity'] ) ) {
				throw new WC_API_Exception( 'woocommerce_api_invalid_product_quantity', __( 'Product quantity is required.', 'woocommerce' ), 400 );
			}
			
			// quantity
			if ( $creating ) {
				$line_item = new WC_Order_Item_Product();
			} else {
				$line_item = new WC_Order_Item_Product( $item['id'] );
			}
			
			$line_item->set_product( $product );
			$line_item->set_order_id( $order->get_id() );
			
			if ( isset( $item['quantity'] ) ) {
				$line_item->set_quantity( $item['quantity'] );
			}
			if ( isset( $item['total'] ) ) {
				$line_item->set_total( floatval( $item['total'] ) );
			} elseif ( $creating ) {
				$total = wc_get_price_excluding_tax( $product, array( 'qty' => $line_item->get_quantity() ) );
				$line_item->set_total( $total );
				$line_item->set_subtotal( $total );
			}
			if ( isset( $item['total_tax'] ) ) {
				$line_item->set_total_tax( floatval( $item['total_tax'] ) );
			}
			if ( isset( $item['subtotal'] ) ) {
				$line_item->set_subtotal( floatval( $item['subtotal'] ) );
			}
			if ( isset( $item['subtotal_tax'] ) ) {
				$line_item->set_subtotal_tax( floatval( $item['subtotal_tax'] ) );
			}
			
			/* @var WC_Order_Item_Product $line_item */
			if( $item['displayname'] ) {
				$line_item->set_name( $item['displayname'] );
			}
			if( $item['description'] ) {
				$line_item->update_meta_data('Description', $item['description']);
			}
			
			if ( $variation_id ) {
				$line_item->set_variation_id( $variation_id );
				$line_item->set_variation( $item['variations'] );
			}
			
			// Save or add to order.
			if ( $creating ) {
				$order->add_item( $line_item );
			} else {
				$item_id = $line_item->save();
				
				if ( ! $item_id ) {
					throw new WC_API_Exception( 'woocommerce_cannot_create_line_item', __( 'Cannot create line item, try again.', 'woocommerce' ), 500 );
				}
			}
		}
	}

